/******************************************************************************
*
*       Fast Reboot setup
*       
******************************************************************************/
#include "stdafx.h"

// instance
HINSTANCE           hInst;
// main window
HWND                hMainDlg;

// NT OS version
//int NTVersion;

//CHAR g_Text[64*1024];
PCHAR g_Text = NULL;

//ULONG Flags = EWX_REBOOT;
//ULONG ShutdownAction = ShutdownReboot;
//BOOL  QuickReboot = TRUE;
//BOOL  UnloadProfile = TRUE;
BOOL  hook_qr = TRUE;
BOOL  hook_rm = TRUE;

BOOL quit = FALSE;
BOOL installed = FALSE;
BOOL not_installed = FALSE;

BOOLEAN reenter = FALSE;

BOOL
SetScriptPath(
    HWND hDlg,
    PCHAR pszInitDir,
    BOOLEAN boot_ini
    );

//----------------------------------------------------------------------
//  
// CenterWindow
// 
// Centers the Window on the screen.
//
//----------------------------------------------------------------------
VOID CenterWindow( HWND hDlg )
{
    RECT            aRt;

    // center the dialog box
    GetWindowRect( hDlg, &aRt );
    OffsetRect( &aRt, -aRt.left, -aRt.top );
    MoveWindow( hDlg,
            ((GetSystemMetrics( SM_CXSCREEN ) -
                aRt.right ) / 2 + 4) & ~7,
            (GetSystemMetrics( SM_CYSCREEN ) -
                aRt.bottom) / 2,
            aRt.right, aRt.bottom, 0 );
} // end CenterWindow()


//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
ULONG
Install(
    HWND hDlg,
    BOOLEAN CopyFiles
    )
{
    HKEY hKey;
    ULONG res;
    CHAR sys_path_name[MAX_PATH];
    CHAR path_name1[MAX_PATH];
    CHAR path_name[MAX_PATH];
    PCHAR p;

    if(check_if_started()) {
        if(!kill_by_caption(L"Fast Reboot Application")) {
            MessageBox( hDlg, "Cannot stop previous running copy of FastReboot", "Error", MB_ICONERROR|MB_OK );
            return FALSE;
        }
        Sleep(500);
    }

    if(CopyFiles) {
        GetSystemDirectory(sys_path_name, MAX_PATH-40);

        _snprintf(path_name, MAX_PATH-1, "%s\\FReboot.exe", sys_path_name);
        GetModuleFileName(NULL, path_name1, MAX_PATH-40);
        res = CopyFile(path_name1, path_name, 0);
        if(!res) {
            res = GetLastError();
            MessageBox( hDlg, "Cannot copy FReboot.exe to System32 directory", "Error", MB_ICONERROR|MB_OK );
            return FALSE;
        }

        _snprintf(path_name, MAX_PATH-1, "%s\\PUHandler.dll", sys_path_name);
        p = strrchr(path_name1, '\\');
        memcpy(p+1, "PUHandler.dll", sizeof("PUHandler.dll"));
        res = CopyFile(path_name1, path_name, 0);
        if(!res) {
            res = GetLastError();
            MessageBox( hDlg, "Cannot copy PUHandler.dll to System32 directory", "Error", MB_ICONERROR|MB_OK );
            return FALSE;
        }
    }

    if(!WriteRegCfgRM()) {
        MessageBox( hDlg, "Cannot store BootDrive information", "Error", MB_ICONERROR|MB_OK );
        return FALSE;
    }

    res = RegOpenKeyEx(HKEY_LOCAL_MACHINE, REG_AUTORUN_STR, 0, KEY_ALL_ACCESS, &hKey);
    if(res != ERROR_SUCCESS) {
        res = GetLastError();
        MessageBox( hDlg, "Cannot save config", "Error", MB_ICONERROR|MB_OK );
        return FALSE;
    }
    if(hook_qr || hook_rm) {
        res = RegSetValueEx(hKey, "FastReboot", NULL, REG_SZ, (PUCHAR)g_Text, strlen(g_Text));
        if(res != ERROR_SUCCESS) {
            res = GetLastError();
            RegCloseKey(hKey);
            MessageBox( hDlg, "Cannot save config", "Error", MB_ICONERROR|MB_OK );
            return FALSE;
        } else {
            if(CopyFiles) {
                res = MessageBox( hDlg, "FastReboot is installed.\n"
                                  "Do you want to apply changes and activate\n"
                                  "Ctrl-Alt-Shift-Del/Break Hot-Keys immediately ?", "Fast Reboot", MB_ICONQUESTION|MB_YESNO);
                if(res == IDYES) {
                    return 2;
                } else {
                    res = MessageBox( hDlg, "New settings shall take effect on next logon", "Fast Reboot", MB_ICONINFORMATION|MB_OK);
                }
            }
        }
        installed = TRUE;
    } else {
        res = RegDeleteValue(hKey, "FastReboot");
        if(res != ERROR_SUCCESS) {
            //res = GetLastError();
            RegCloseKey(hKey);
            if(res == 2) {
                MessageBox( hDlg, "FastReboot not installed", "Fast Reboot", MB_ICONEXCLAMATION|MB_OK );
            } else {
                MessageBox( hDlg, "Cannot deinstall FastReboot", "Error", MB_ICONERROR|MB_OK );
            }
            return FALSE;
        } else {
            MessageBox( hDlg, "FastReboot deinstalled", "Fast Reboot", MB_ICONEXCLAMATION|MB_OK );
        }
        installed = FALSE;
    }
    RegCloseKey(hKey);

    return TRUE;
} // end Install()

//----------------------------------------------------------------------
//  
// RefreshItems
//
// Update controls.
//
//----------------------------------------------------------------------
void
RefreshItems(
    HWND hDlg
    )
{
    ULONG i;

    i=0;
    memcpy(g_Text+i, "FReboot.exe -", sizeof("FReboot.exe -"));
    i += sizeof("FReboot.exe -")-1;

    if(hook_rm) {
        g_Text[i] = 'm';
        i++;
    }
    if(Flags & (EWX_REBOOT | EWX_SHUTDOWN | EWX_POWEROFF)) {
        if(hook_qr) {
            g_Text[i] = 'q';
            i++;
        }
        switch(ShutdownAction) {
        case ShutdownReboot:
            SendDlgItemMessage(hDlg, IDC_ACTION_REBOOT, BM_SETCHECK, BST_CHECKED, 0L);
            g_Text[i] = 'r';
            break;
        case ShutdownNoReboot:
            SendDlgItemMessage(hDlg, IDC_ACTION_QR_SHUTDOWN, BM_SETCHECK, BST_CHECKED, 0L);
            g_Text[i] = 's';
            break;
        case ShutdownPowerOff:
            SendDlgItemMessage(hDlg, IDC_ACTION_QR_POWEROFF, BM_SETCHECK, BST_CHECKED, 0L);
            g_Text[i] = 'p';
            break;
        }
    } else {
        SendDlgItemMessage(hDlg, IDC_ACTION_LOGOFF, BM_SETCHECK, BST_CHECKED, 0L);
        QuickReboot = FALSE;
        UnloadProfile = TRUE;
        g_Text[i] = 'l';
    }
    i++;
    if(!QuickReboot) {
        SendDlgItemMessage(hDlg, IDC_QR_NORMAL, BM_SETCHECK, BST_CHECKED, 0L);
        SendDlgItemMessage(hDlg, IDC_QR_SAVE_PROF, BM_SETCHECK, BST_UNCHECKED, 0L);
        SendDlgItemMessage(hDlg, IDC_QR_QUICK, BM_SETCHECK, BST_UNCHECKED, 0L);
    } else {
        if(UnloadProfile) {
            SendDlgItemMessage(hDlg, IDC_QR_NORMAL, BM_SETCHECK, BST_UNCHECKED, 0L);
            SendDlgItemMessage(hDlg, IDC_QR_QUICK, BM_SETCHECK, BST_UNCHECKED, 0L);
            SendDlgItemMessage(hDlg, IDC_QR_SAVE_PROF, BM_SETCHECK, BST_CHECKED, 0L);
            g_Text[i] = 'u';
            i++;
        } else {
            SendDlgItemMessage(hDlg, IDC_QR_NORMAL, BM_SETCHECK, BST_UNCHECKED, 0L);
            SendDlgItemMessage(hDlg, IDC_QR_SAVE_PROF, BM_SETCHECK, BST_UNCHECKED, 0L);
            SendDlgItemMessage(hDlg, IDC_QR_QUICK, BM_SETCHECK, BST_CHECKED, 0L);
            Flags |= EWX_FORCE;
        }
    }
    SendDlgItemMessage(hDlg, IDC_CHECK_FORCE, BM_SETCHECK, (Flags & EWX_FORCE) ? BST_CHECKED : BST_UNCHECKED, 0L);
    if(Flags & EWX_FORCE) {
        g_Text[i] = 'f';
        i++;
    }
    SendDlgItemMessage(hDlg, IDC_CHECK_REBOOTMGR, BM_SETCHECK, hook_rm ? BST_CHECKED : BST_UNCHECKED, 0L);
    SendDlgItemMessage(hDlg, IDC_CHECK_QUICKREBOOT, BM_SETCHECK, hook_qr ? BST_CHECKED : BST_UNCHECKED, 0L);

    if(ShutdownScript[0]) {
        memcpy(g_Text+i, " --script \"", sizeof(" --script \""));
        i += sizeof(" --script \"")-1;
        if(ShutdownScript[0] == '\"') {
            strcpy(g_Text+i, ShutdownScript+1);
            i += strlen(ShutdownScript+1);
        } else {
            strcpy(g_Text+i, ShutdownScript);
            i += strlen(ShutdownScript);
        }
        if(g_Text[i-1] != '\"') {
            g_Text[i] = '\"';
            i++;
        }
    }

    g_Text[i] = 0;
    i++;

    return;

} // end RefreshItems()

//----------------------------------------------------------------------
//  
// RefreshItems
//
// Update controls.
//
//----------------------------------------------------------------------
void
GetItems(
    HWND hDlg
    )
{
    ULONG d;

    if(reenter)
        return;

    Flags = 0;
    ShutdownAction = ShutdownReboot;
    QuickReboot = FALSE;
    
    d = SendDlgItemMessage(hDlg, IDC_ACTION_REBOOT, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        Flags |= EWX_REBOOT;
        ShutdownAction = ShutdownReboot;
    }
    d = SendDlgItemMessage(hDlg, IDC_ACTION_QR_SHUTDOWN, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        Flags |= EWX_SHUTDOWN;
        ShutdownAction = ShutdownNoReboot;
    }
    d = SendDlgItemMessage(hDlg, IDC_ACTION_QR_POWEROFF, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        Flags |= EWX_POWEROFF;
        ShutdownAction = ShutdownPowerOff;
    }
    d = SendDlgItemMessage(hDlg, IDC_ACTION_LOGOFF, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        ShutdownAction = ShutdownReboot;
        Flags = EWX_LOGOFF;
    }

    if(Flags != EWX_LOGOFF) {
        d = SendDlgItemMessage(hDlg, IDC_QR_QUICK, BM_GETCHECK, 0, 0L);
        if(d == BST_CHECKED) {
            QuickReboot = TRUE;
            UnloadProfile = FALSE;
            Flags |= EWX_FORCE;
        }
    }
    d = SendDlgItemMessage(hDlg, IDC_QR_SAVE_PROF, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        UnloadProfile = TRUE;
        QuickReboot = TRUE;
    }
    d = SendDlgItemMessage(hDlg, IDC_QR_NORMAL, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        QuickReboot = FALSE;
    }

    d = SendDlgItemMessage(hDlg, IDC_CHECK_FORCE, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        Flags |= EWX_FORCE;
    }

    d = SendDlgItemMessage(hDlg, IDC_CHECK_REBOOTMGR, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        hook_rm = TRUE;
    } else {
        hook_rm = FALSE;
    }

    d = SendDlgItemMessage(hDlg, IDC_CHECK_QUICKREBOOT, BM_GETCHECK, 0, 0L);
    if(d == BST_CHECKED) {
        hook_qr = TRUE;
    } else {
        hook_qr = FALSE;
    }

    if(!hook_qr && !hook_rm) {
        EnableWindow(GetDlgItem(hDlg, IDINSTALL), FALSE);
    } else {
        EnableWindow(GetDlgItem(hDlg, IDINSTALL), TRUE);
    }

    GetDlgItemText(hDlg, IDC_REBOOT_SCRIPT, ShutdownScript, MAX_PATH-1);

    GetDlgItemText(hDlg, IDC_BOOT_INI, (PCHAR)BootIniPath, sizeof("x:\\boot.ini"));
    BootDrv[0] = BootIniPath[0];

    return;

} // end GetItems()

//----------------------------------------------------------------------
//  
// MainDialog
//
// This is the main window. 
//
//----------------------------------------------------------------------

LONG
APIENTRY
MainDialog(
    HWND hDlg,
    UINT message,
    UINT wParam,
    LONG lParam
    )
{
    switch (message) {
    case WM_INITDIALOG :

        UCHAR a;
        // center the window
        CenterWindow( hDlg );
        RefreshItems(hDlg);
        reenter = TRUE;
        SetDlgItemText(hDlg, IDC_REBOOT_SCRIPT, ShutdownScript);
        //SetDlgItemText(hDlg, IDC_BOOT_INI, (PCHAR)BootIniPath);
        a = BootIniPath[0];
        FindBootIni(GetDlgItem(hDlg, IDC_BOOT_INI));
        BootIniPath[0] = a;
        SendMessage(GetDlgItem(hDlg, IDC_BOOT_INI), CB_SELECTSTRING, -1, (LPARAM)BootIniPath);
        BootDrv[0] = BootIniPath[0];
        reenter = FALSE;
        if(not_installed) {
            EnableWindow(GetDlgItem(hDlg, IDDEINSTALL), FALSE);
        }
        ShowWindow(hDlg, TRUE);
        return FALSE;

    case WM_COMMAND:
        switch (LOWORD( wParam )) {
        case IDAPPLY:
            if(Install(hDlg, FALSE)) {
                if(hook_rm || hook_qr) {
                    EnableWindow(GetDlgItem(hDlg, IDDEINSTALL), TRUE);
                    //EnableWindow(GetDlgItem(hDlg, IDINSTALL), FALSE);
                } else {
                    EnableWindow(GetDlgItem(hDlg, IDDEINSTALL), FALSE);
                    EnableWindow(GetDlgItem(hDlg, IDINSTALL), TRUE);
                }
                EndDialog (hDlg, 0);
                PostQuitMessage( 0 );
            }
            break;

        case IDINSTALL:
            //hook_rm = TRUE;
            //hook_qr = TRUE;
            ULONG res;
            RefreshItems(hDlg);
            res = Install(hDlg, TRUE);
            if(res == 2) {
                EndDialog (hDlg, 0);
                PostQuitMessage( 0 );
            }
            if(res) {
                EnableWindow(GetDlgItem(hDlg, IDDEINSTALL), TRUE);
            }
            break;

        case IDDEINSTALL:
            hook_rm = FALSE;
            hook_qr = FALSE;
            RefreshItems(hDlg);
            if(Install(hDlg, FALSE)) {
                EnableWindow(GetDlgItem(hDlg, IDDEINSTALL), FALSE);
                EnableWindow(GetDlgItem(hDlg, IDINSTALL), TRUE);
            }
            break;

        case IDCANCEL:
            quit = TRUE;
            installed = FALSE; // prevent running in background
            EndDialog (hDlg, 0) ;
            PostQuitMessage (0) ;
            break ;

        case IDC_SELECT_SCRIPT:
            if(SetScriptPath(hDlg, NULL, FALSE)) {
                SetDlgItemText(hDlg, IDC_REBOOT_SCRIPT, ShutdownScript);
                RefreshItems(hDlg);
            } else {
                GetDlgItemText(hDlg, IDC_REBOOT_SCRIPT, ShutdownScript, MAX_PATH-1);
            }
            break ;

        default:
            GetItems(hDlg);
            RefreshItems(hDlg);
        }
        break; 

    case WM_CLOSE:  
        quit = TRUE;
        EndDialog (hDlg, 0);
        PostQuitMessage( 0 );
        break;
    }
    //return FALSE;
    return DefWindowProc ( hDlg, message, wParam, lParam);
} // end MainDialog()


//----------------------------------------------------------------------
//
//----------------------------------------------------------------------
UINT_PTR CALLBACK MyHook2(HWND hdlg, UINT uiMsg, WPARAM wParam, LPARAM lParam)
{
    return TRUE;
}

BOOL
SetScriptPath(
    HWND hDlg,
    PCHAR pszInitDir,
    BOOLEAN boot_ini
    )
{
    CHAR szFile[MAX_PATH] = "\0";
    PCHAR pszLogPath;
    BOOLEAN free_buff = FALSE;
    //pszLogPath = (PCHAR) GlobalAlloc(GMEM_FIXED, sizeof(CHAR)*MAX_PATH);
    pszLogPath = boot_ini ? (PCHAR)BootIniPath : ShutdownScript;
    //pszFile = (PCHAR) malloc (sizeof(CHAR)*MAX_PATH);
    //pszFile[0] = '\0';
    //memcpy(pszFile, "DbgDump.log", sizeof("DbgDump.log"));
    CHAR szFilterExe[MAX_PATH] = "Executables (*.exe,*.bat,*.cmd)\0*.exe;*.bat;*.cmd\0"
                              "All files (*.*)\0*.*";
    CHAR szFilterBootIni[MAX_PATH] = "boot.ini\0boot.ini";
    PCHAR szFilter = boot_ini ? szFilterBootIni : szFilterExe;
    CHAR szFileTitle[MAX_PATH];

    if(pszInitDir == NULL)
    {
        pszInitDir = (PCHAR)GlobalAlloc(GMEM_FIXED, sizeof(CHAR)*MAX_PATH);
        DWORD dwDirRes = GetCurrentDirectory(MAX_PATH, pszInitDir);
        if(!dwDirRes) // error
        {
            GlobalFree(pszInitDir);
            pszInitDir = NULL;
        } else {
            free_buff = TRUE;
        }
    }
    OPENFILENAME openfilename;

#ifndef OFN_DONTADDTORECENT
#define OFN_DONTADDTORECENT          0x02000000
#define OFN_FORCESHOWHIDDEN          0x10000000    // Show All files including System and hidden files
#endif //OFN_DONTADDTORECENT

    memset(&openfilename,0,sizeof(OPENFILENAME));
	openfilename.lStructSize = sizeof(OPENFILENAME_NT4);
	openfilename.hwndOwner = hDlg;
	openfilename.hInstance = hInst; 
	openfilename.lpstrFilter = szFilter;
	openfilename.lpstrCustomFilter = NULL;
	openfilename.nMaxCustFilter = 0;
	openfilename.nFilterIndex = 0;
    openfilename.lpstrFile = szFile;
    openfilename.nMaxFile = MAX_PATH;
	openfilename.lpstrFileTitle = szFileTitle;
	openfilename.nMaxFileTitle = sizeof(szFileTitle);
	openfilename.lpstrInitialDir = pszInitDir;
	openfilename.lpstrTitle = boot_ini ? (LPSTR)"Locate active BOOT.INI" : (LPSTR)"Set pre-reboot script path...";
	openfilename.Flags = OFN_EXPLORER | OFN_ENABLEHOOK | 
	    (boot_ini ? (OFN_NONETWORKBUTTON | OFN_FORCESHOWHIDDEN | OFN_DONTADDTORECENT) :
	                (OFN_HIDEREADONLY)) ;
    openfilename.nFileOffset = 0;
	openfilename.nFileExtension = 0;
	openfilename.lpstrDefExt = NULL;
	openfilename.lCustData = 0;	
    openfilename.lpfnHook = MyHook2;
    openfilename.lpTemplateName = NULL; 
    
    BOOL bRet = GetOpenFileName(&openfilename);
    DWORD dwRes = CommDlgExtendedError();
    if(free_buff) {
        GlobalFree(pszInitDir);
    }
    memcpy(pszLogPath, openfilename.lpstrFile, boot_ini ? sizeof("x:\\boot.ini") : MAX_PATH);
    return bRet; 

} // end SetScriptPath()


//----------------------------------------------------------------------
//  
// WinMain
//
//----------------------------------------------------------------------
int
//CALLBACK
FRSetup(
    HINSTANCE hInstance,
    int nCmdShow
    )
{     
    static TCHAR    szAppName [] = TEXT("FastRebootSetup") ;
    MSG             msg ;
    WNDCLASSEX      wndclass ;

    // save instance
    hInst = hInstance;
    // init common controls
    InitCommonControls();

    g_Text = (PCHAR)GlobalAlloc(GMEM_FIXED, 64*1024);

/*
    // get NT version
    NTVersion = GetVersion();
    if( NTVersion >= 0x80000000 ) {
        return TRUE;
    }
*/

    // create the main window class
    wndclass.cbSize         = sizeof( WNDCLASSEX );
    wndclass.style          = CS_HREDRAW | CS_VREDRAW ;
    wndclass.lpfnWndProc    = (WNDPROC) MainDialog ;
    wndclass.cbClsExtra     = 0 ;
    wndclass.cbWndExtra     = DLGWINDOWEXTRA ;
    wndclass.hInstance      = hInstance ;
    wndclass.hIcon          = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_APP_ICON));
    wndclass.hIconSm        = LoadIcon (hInstance, MAKEINTRESOURCE(IDI_APP_ICON));
    wndclass.hCursor        = LoadCursor (NULL, IDC_ARROW) ;
    wndclass.hbrBackground  = (HBRUSH) (COLOR_BTNFACE+1);
    wndclass.lpszMenuName   = NULL ;
    wndclass.lpszClassName  = szAppName ;
    RegisterClassEx (&wndclass) ;
 
    // create the dialog
    hMainDlg = CreateDialog( hInstance, "FastRebootSetup", NULL,/*hDummyWnd,*/ (DLGPROC)MainDialog) ;
    ShowWindow(hMainDlg, TRUE);
 
    while (GetMessage (&msg, NULL, 0, 0)) {
        if( !IsDialogMessage( hMainDlg, &msg )) {
          TranslateMessage (&msg) ;
          DispatchMessage (&msg) ;
        }
    }

    if(quit && !installed)
        ExitProcess(0);

    GlobalFree(g_Text);

    return msg.wParam;
}


